/* This file is part of GNOME Games. License: GPL-3.0+. */

.savestate-row {
	padding: 0;
}

.savestate-thumbnail {
	color: rgba(255, 255, 255, 0.5);
	background: rgba(0, 0, 0, .5);
	border: 1px solid rgba(0, 0, 0, .5);
	border-radius: 5px;
}

.savestate-name {
	font-size: large;
}

.savestate-date {
	opacity: 0.5;
	font-size: small;
}

.savestates-list {
	border-left: none;
	border-right: none;
}

gamesgamethumbnail {
	background-color: mix(@theme_base_color, @theme_bg_color, 0.5);
	border-width: 1px;
	border-style: solid;
	border-radius: 2px;
	border-color: @theme_bg_color;
	color: @theme_fg_color;
}

gamescollectioniconview flowboxchild {
	min-width: 128px;
}

gamescollectioniconview.large flowboxchild {
	min-width: 256px;
}

gamesflashbox {
  background: rgba(255, 255, 255, 0.5);
}

/* Styles from libhandy example, this should be kept in sync with this:
 * https://source.puri.sm/Librem5/libhandy/blob/master/examples/style.css
 */

.list-icon {
	opacity: 0.7;
}

.combo-list-box {
	background: none;
}

box.sidebar,
box.sidebar > list {
	transition: all 200ms;
	background: @theme_base_color;
	border: 0 solid @borders;
}

box.sidebar:backdrop,
box.sidebar:backdrop > list {
	background-color: @theme_unfocused_base_color;
	border-color: @unfocused_borders;
}

.folded box.sidebar {
	padding: 18px;
	background: @theme_bg_color;
}

.folded:backdrop box.sidebar {
	background: @theme_unfocused_bg_color;
}
list.rounded,
.folded box.sidebar > list {
	background: @theme_base_color;
	border-radius: 6px;
	border: 1px solid @borders;
}

list.rounded row:first-child,
	.folded box.sidebar > list row:first-child {
	border-top-left-radius: 6px;
	border-top-right-radius: 6px;
	-gtk-outline-top-left-radius: 6px;
	-gtk-outline-top-right-radius: 6px;
}

list.rounded row:last-child,
.folded box.sidebar > list row:last-child {
	border-bottom-left-radius: 6px;
	border-bottom-right-radius: 6px;
	-gtk-outline-bottom-left-radius: 6px;
	-gtk-outline-bottom-right-radius: 6px;
}

list.rounded:backdrop,
.folded:backdrop box.sidebar > list {
	background: @theme_unfocused_base_color;
	border-color: @unfocused_borders;
}

list.separators row:not(:last-child) {
	border-bottom: 1px solid rgba(0, 0, 0, 0.15);
}
